
# q-expansion of newform 637.2.q.g, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 637
weight = 2
poly_data = [64, -96, 16, 80, -60, -20, 45, -10, -15, 10, 1, -3, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[-800, 608, 344, -620, 42, 313, -164, -91, 72, -9, -13, 1], 224], [[256, -320, 40, 260, -154, -73, 132, -21, -32, 25, 5, -9], 224], [[288, -304, -88, 380, -140, -167, 166, 21, -78, 29, 17, -11], 224], [[544, -512, -160, 584, -126, -289, 200, 91, -96, 5, 29, -13], 224], [[-464, 216, 260, -340, 7, 194, -101, -42, 51, -9, -13, 8], 112], [[-1664, 1184, 720, -1452, 42, 821, -396, -231, 208, -5, -57, 13], 224], [[-88, 96, 16, -99, 35, 45, -41, -7, 18, -4, -5, 2], 28], [[-48, 102, 3, -82, 56, 43, -44, 0, 20, -6, -4, 3], 28], [[464, -664, -64, 634, -273, -271, 269, 21, -121, 30, 20, -15], 112], [[1056, -872, -500, 1062, -168, -547, 310, 133, -160, 13, 39, -17], 112]]

hecke_ring_character_values = [[248, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [197, [1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0], [0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0], [0, 1, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [-1, -1, 1, 0, 0, 1, 0, 0, 1, 0, -1, 0], [-2, 2, -1, -1, -1, 2, -2, 0, 0, -1, 1, -1], [3, 1, 0, 1, 2, 0, -1, 1, -1, 0, 0, -1], [0, 0, 0, 0, 1, 0, 1, 1, -1, 1, -1, 1], [-1, 1, -2, -1, -1, 1, 0, 1, -1, -2, 1, 1], [3, -2, 1, 2, 1, -3, 1, 2, -1, 0, 0, 2], [-1, -2, -1, -2, -3, -1, -2, 1, -1, -2, 0, 1], [-1, -1, 1, 1, 1, -1, 2, -2, 2, 0, -3, 2], [0, -3, 0, -1, -1, 2, 1, 2, -1, 0, 1, -2], [2, 0, 0, 1, 4, 0, 3, -2, 3, -2, -2, 2], [-1, 1, 0, 1, -2, 0, 0, 0, -1, 0, 0, 0], [3, -2, 2, 2, 2, -2, 1, -2, 3, 0, 0, 2], [-3, 5, 0, -4, 3, 0, 1, 0, -1, -4, 4, 0], [1, 0, 2, 1, 1, 2, -1, 0, 1, 0, 0, -2], [-3, 0, -1, -4, -3, 1, -1, 2, -5, 0, -1, -2], [1, 1, 0, -3, -1, 0, -2, 0, 2, 3, -3, 0], [-4, 4, 0, 2, -4, 2, 2, -2, 0, 1, 0, 0], [6, 0, 0, 1, 0, -1, 4, -1, -3, 1, -2, 0], [-3, 4, -3, -1, 1, 2, 3, -2, 1, -3, 0, 3], [-6, 0, -1, -2, -5, 2, -2, 3, -3, 0, 4, -3], [-1, 0, -1, 3, -2, 1, 1, -1, -2, 2, -2, -2], [3, -3, 2, 3, -3, -3, 0, 1, -1, 4, -2, 1], [3, 0, 2, -1, 2, -1, -3, -1, 4, 1, -2, 2], [-1, -4, 0, -2, -1, 1, -5, -1, 2, -2, 1, -1], [3, 2, -3, 1, 5, -2, 3, 2, -1, -2, 0, 3], [-4, 2, 0, -5, -6, 0, -9, 2, -9, 2, 2, -2], [-2, -4, -3, -4, 0, 2, -6, 1, 1, 0, 0, 1], [3, 3, -4, 0, -4, 1, -4, 1, 0, -2, 4, -4], [-2, -2, -2, -2, -1, 2, -5, 1, 3, 4, -4, -1], [0, -3, 5, 1, 2, 5, 1, -2, 6, 0, 0, -3], [-4, -2, 1, 3, 6, -1, 2, 0, -1, 3, -3, 1], [0, 1, 2, -1, 6, 4, 1, -4, 4, -1, 0, -2], [7, -1, -2, -2, -2, 5, -2, 5, -2, -3, 6, -2], [-7, 5, -2, -3, 4, 2, 1, 1, -3, -2, 2, -1], [-2, -1, 4, -4, 3, 0, 1, -4, 0, 0, 4, 4], [3, -2, 4, -2, 6, 4, -1, -3, 3, -5, -5, -1], [0, -5, 2, 1, -4, 0, -5, -2, 1, 0, 0, -2], [-2, 0, 0, 3, 0, -3, 4, -3, -1, -1, 2, 0], [4, -3, -1, -4, 1, -1, -3, 3, -4, 1, 1, -2], [-1, 2, -3, -5, -2, -3, 1, 0, -8, 0, 3, 0], [-11, 5, -2, 1, -8, 1, -6, 3, -1, 0, 3, -3], [9, -2, 1, 0, 9, 1, 1, 0, 2, -1, -1, -1], [-5, 3, 1, -2, -9, 5, -2, -6, 3, -4, 2, -6], [9, 3, -8, 5, -2, -3, 0, 5, -3, 0, -5, -5], [-5, 0, -3, -2, -2, 3, -6, -1, 3, 3, -3, -4], [1, -2, -5, -6, 5, -1, -3, 1, -2, 1, 0, 5], [0, 3, -2, -1, -2, 0, -9, 0, 6, 2, -4, -2], [3, 4, -3, 2, 7, -1, 3, 1, -2, -1, 0, 3], [-4, 1, 2, 5, 2, -2, 2, -6, 0, -1, 1, -4], [9, 0, -1, -3, 6, -1, -5, 3, -6, -3, -3, -2], [5, 1, 5, 9, -1, -5, 6, 0, -4, -2, 1, 0], [4, 2, 5, 8, -2, -4, 6, -1, -3, 4, -2, -1], [7, -1, -4, 4, -3, -4, -1, 10, -5, 2, 2, -6], [-4, -1, 3, -2, 3, -4, 5, -7, 1, 0, 1, 7], [-10, 0, 3, 7, -10, 3, 4, 0, 7, -3, -3, -3], [11, 1, 4, 9, 10, -4, -1, 4, -6, 8, 0, -4], [-3, 5, -6, -2, -5, 3, 2, 3, -4, 4, -2, 3], [-12, 11, -4, -7, 4, 4, 0, 0, -4, -1, 1, -4], [0, 9, 2, 14, -8, -3, 11, 3, -1, -1, 0, -2], [-14, 1, -3, -4, -3, 6, -5, 6, -2, -1, 2, -3], [5, -6, 3, 1, 3, 2, 4, 2, 0, -3, 6, 3], [8, -1, -2, 3, 6, -6, 0, 6, -5, 8, 0, 2], [-1, -2, -4, -4, -5, 4, -8, 2, 4, 0, 0, -2], [3, -11, 2, 8, 2, 1, -3, 1, 13, 5, -10, 2], [-9, 6, -3, -3, -5, -3, -2, -4, -5, 3, 3, 7], [-14, 2, 3, -1, -3, -2, 0, -5, 2, 0, -4, 5], [9, -1, 2, 2, 5, 3, -2, 1, 4, 0, 0, -1], [-5, 12, -2, 10, -8, 0, 9, 0, -3, -1, 0, 2], [-3, 4, 0, -3, 6, 3, 1, -3, 3, -4, 2, -3], [8, -4, 1, 4, 5, 1, 4, 3, 5, -1, -1, -4], [-12, 1, -5, -4, -11, 0, -1, 5, -9, 0, 0, -5], [17, -9, 15, 9, 9, -15, 11, -4, 4, 2, -2, 11], [9, -11, 8, 3, 8, 0, -2, 0, 13, 1, -2, 8], [-1, 4, -6, -3, -10, 6, -4, 1, -2, -4, 4, -5], [-13, 13, -2, -12, -7, -3, -10, -1, -14, 0, 5, 1], [12, -6, 5, 0, -2, -5, -1, 0, 6, 6, -6, 5], [5, -8, 9, 12, -1, -5, -3, -4, -3, 12, -6, -4], [-2, 2, 2, 2, -2, 2, 3, -2, 3, 5, 0, -2], [-14, 8, 0, -9, -7, 0, -8, 0, -9, 0, -4, 0], [-4, -8, 3, -9, -6, 3, -6, 2, -3, 4, 4, -5], [-8, -6, -4, -3, -10, 8, -14, -4, -1, 8, -4, -4], [-1, 12, -11, -6, -11, 5, -9, 5, -8, 1, -2, -11], [20, -15, 9, 5, -4, -9, -3, -2, 12, 4, -4, 7], [9, 3, -4, 0, -4, 9, -12, 13, -4, 0, 7, -13], [0, -4, 0, -3, 3, 0, -4, 3, 4, -11, 11, 3], [5, -5, 3, -4, 11, 2, -1, -2, 6, -2, 0, -3], [-12, 6, -3, -9, -3, 6, 2, 6, -14, -5, 10, -3], [-4, 3, -8, 2, -7, -6, 3, 2, -6, 0, 5, -2], [-7, -4, -1, 4, -1, 1, -7, -6, 6, 10, -10, -7], [0, -5, -1, -7, 17, 0, -5, 1, 6, 2, -1, 1], [-6, 5, -6, -4, 0, 3, 9, -3, 7, -6, 0, 6], [-12, 3, -2, 0, -13, -2, -2, 1, -4, 5, 5, 1], [0, 5, -3, 4, 0, -6, 1, -3, 1, 0, -11, 3], [1, 6, 3, -8, 3, -1, 10, -1, -15, 5, -10, 3], [-4, -4, 0, -7, 11, 4, -8, -4, 7, 6, -3, -4], [-6, -3, 3, 2, -19, -2, -7, 2, -6, 7, 0, -3], [-2, 11, -6, -11, -6, 6, -8, 6, -9, -6, 12, -6], [-2, 11, 1, -12, -3, 6, -17, 5, -11, 0, 10, -5], [4, -11, 0, -8, 2, 0, 1, 2, 1, -3, -3, -2], [10, -6, 2, 4, -6, -10, 4, 8, -2, -6, 3, 8], [-1, 4, -19, 5, -19, 10, -12, 10, -2, -3, 6, -19], [-1, -2, -5, -1, -9, -6, 1, 6, -3, -2, 0, 5], [2, 0, 9, -4, 8, 4, -4, -5, 5, 0, -11, 5], [5, 5, 2, 0, 22, 7, 7, -7, 9, -1, 0, -2], [-10, 0, 8, -7, 8, -1, 5, -1, -4, -2, 4, 8], [6, -8, 3, 2, -3, -6, -2, 3, 1, 20, -10, 3], [-9, 3, 5, 0, -11, 5, -10, 2, -5, 2, 2, -7], [-6, 12, -4, -9, -4, -6, -6, -2, -13, 0, -1, 2], [5, -1, 5, 0, 5, -5, 4, 0, 1, -1, 1, 5], [-3, 1, -2, 0, -6, -1, -7, 1, -9, -10, 0, 2], [3, -7, -2, 9, -8, 2, -1, -1, -1, 2, -2, -3], [4, 2, -7, 6, -3, -7, 4, 7, -3, -3, -3, 0], [4, 2, 4, 0, -2, -4, 0, -6, 4, 2, -2, -2], [-13, 7, -5, -10, -11, 13, -6, -8, 5, -18, 9, -8], [6, -9, 6, -4, 6, -6, -3, 0, 10, -6, 3, 0], [6, -3, -3, 12, -2, -3, 10, 8, 7, -3, -3, -5], [1, -2, 12, 2, 15, -5, 7, -17, 14, 0, -10, 17], [-1, -1, -5, -8, 1, 1, -2, 4, 3, -12, 6, 4], [12, -4, 4, 2, 4, -2, -3, -2, 9, 6, -12, 4], [5, -8, 6, -2, 4, -6, -7, -3, 13, -1, 1, 3], [-7, -4, -2, -2, -7, 3, 1, 5, -4, 0, 1, -5], [1, -5, -1, 0, -1, 6, -2, 6, 1, -1, 2, -1], [5, -1, 12, 12, 6, -12, 10, -13, 2, 0, 0, -1], [15, -3, 2, -8, 14, 2, -8, 1, -6, 8, 8, -3], [2, 12, -6, -4, -6, -2, -1, -2, -9, -5, 10, -6], [-4, -11, 2, -6, -16, -3, -5, 3, 3, -2, 0, -2], [2, 0, -8, 9, -10, 6, -6, 14, 1, 0, 2, -14], [-9, 6, 10, -15, 6, -1, -5, -11, -5, 0, 0, 11], [-6, 9, -2, -9, 1, 6, 3, -4, 7, 12, -6, -4], [4, 1, 7, 6, 8, -4, 5, -3, 1, -4, 2, -3], [4, -16, 7, 7, 6, -7, -6, -4, 13, 19, -19, 3], [-3, -10, -1, -2, 0, -5, 15, -4, -3, 0, -7, 4], [13, 2, 13, 10, 1, -13, 13, -12, 0, -5, 5, 1], [9, -12, 13, 8, 19, -13, 11, 2, 2, 10, -10, 15], [7, 5, -6, 12, -6, -6, 0, 13, -6, -6, -6, -7], [4, -1, 8, 16, -13, -4, 3, -4, -8, 16, -8, -4], [-1, 3, 5, 8, -7, 0, -1, 0, -4, 2, 0, -5], [3, -2, -1, 5, 2, -3, 5, -2, 4, 0, 1, 2], [6, 5, 1, 3, 15, 1, 6, -9, 7, -8, -8, 8], [-10, 11, 5, 2, 3, 14, 0, -14, 3, -6, 0, -5], [-4, 4, -2, -11, 8, 4, 0, -2, 9, 18, -9, -2], [3, -7, 10, 4, 13, -10, 3, -4, 7, 5, -5, 6], [-4, -2, -3, -11, 7, 6, -18, -6, -10, 3, 0, 3], [7, 2, -9, 0, -9, 7, -6, 7, -3, -1, 2, -9], [12, 5, 0, -3, 0, -2, 2, -2, -5, -7, 14, 0], [9, -6, 11, 1, 15, 4, -8, -4, 2, 8, 0, -11], [-5, -1, -1, 6, -5, 1, -4, -8, 3, 5, -5, -9], [-8, -2, 3, -10, -10, 8, -10, -11, 13, -2, 1, -11], [-6, 2, 5, -9, 5, 2, -5, 2, 1, -7, 14, 5], [-8, 26, 2, 14, 3, 8, 18, -10, -12, 0, 0, -10], [-5, -5, 2, -12, -18, 5, -10, -7, 14, 4, -2, -7], [13, -13, 10, 13, 10, -10, 21, -10, 2, 9, -18, 10], [-6, 11, -1, 7, -6, -1, -3, 0, -4, -11, -11, 1], [8, 14, 1, -6, -7, -1, 8, -1, -7, -7, 7, 0], [7, -9, 6, 4, 6, -1, -5, -1, 15, -3, 6, 6], [-13, 4, -8, -15, 4, 11, 1, -11, 8, -1, 0, 8], [-1, -5, -1, 14, 4, -11, 16, -10, 13, 0, -2, 10], [15, 3, -7, 8, 8, -7, 5, 7, -2, 3, 3, 0], [-16, -4, 0, -3, -34, -1, -6, 1, -3, 13, 0, 0], [-16, 7, -2, 7, -31, -2, -13, 15, -15, 2, 2, -13], [7, -9, 6, -3, 2, 15, -6, 9, 3, 0, -9, -9], [7, 3, 4, 5, 14, 2, -5, -2, -6, 3, 0, -4], [6, -6, 5, 15, 5, -6, 0, 1, -10, 18, -9, 1], [1, 11, -7, 10, -21, -7, -16, 22, -23, 0, 0, -15]]
